@@ -89,7 +89,6 @@ def objectify_response_json(response):
 
 
 class Page(object):
-
     endpoint = ''
 
     def __init__(self, connection=None, *a, **kw):
@@ -154,6 +153,26 @@ class Page(object):
         resp.status_code = 200
         return cls(r=resp, connection=connection)
 
+    @property
+    def bytes(self):
+        if self.r is None:
+            return b''
+        return self.r.content
+
+    def extract_data(self, response):
+        """Takes a `requests.Response` and returns a data dict."""
+        try:
+            data = response.json()
+        except ValueError as e:  # If there was no json to parse
+            data = {}
+            if response.text or response.status_code not in (200, 202, 204):
+                text = response.text
+                if len(text) > 1024:
+                    text = text[:1024] + '... <<< Truncated >>> ...'
+                log.debug("Unable to parse JSON response ({0.status_code}): {1} - '{2}'".format(response, e, text))
+
+        return data
+
     def page_identity(self, response, request_json=None):
         """Takes a `requests.Response` and
         returns a new __item_class__ instance if the request method is not a get, or returns
@@ -171,16 +190,7 @@ class Page(object):
         else:
             ds = None
 
-        try:
-            data = response.json()
-        except ValueError as e:  # If there was no json to parse
-            data = dict()
-            if response.text or response.status_code not in (200, 202, 204):
-                text = response.text
-                if len(text) > 1024:
-                    text = text[:1024] + '... <<< Truncated >>> ...'
-                log.debug("Unable to parse JSON response ({0.status_code}): {1} - '{2}'".format(response, e, text))
-
+        data = self.extract_data(response)
         exc_str = "%s (%s) received" % (http.responses[response.status_code], response.status_code)
 
         exception = exception_from_status_code(response.status_code)
@@ -188,7 +198,6 @@ class Page(object):
             raise exception(exc_str, data)
 
         if response.status_code in (http.OK, http.CREATED, http.ACCEPTED):
-
             # Not all JSON responses include a URL.  Grab it from the request
             # object, if needed.
             if 'url' in data:
@@ -326,7 +335,6 @@ def exception_from_status_code(status_code):
 
 
 class PageList(object):
-
     NATURAL_KEY = None
 
     @property
@@ -487,6 +495,7 @@ class TentativePage(str):
 class PageCache(object):
     def __init__(self):
         self.options = {}
+        self.post_fields = {}
         self.pages_by_url = {}
         self.pages_by_natural_key = {}
 
@@ -508,6 +517,29 @@ class PageCache(object):
 
         return self.options.setdefault(url, options)
 
+    def get_post_fields(self, page):
+        url = page.endpoint if isinstance(page, Page) else str(page)
+        key = get_registered_page(url)
+        if key in self.post_fields:
+            return self.post_fields[key]
+
+        options_page = self.get_options(page)
+
+        if options_page is None:
+            return None
+
+        if 'POST' not in options_page.r.headers.get('Allow', ''):
+            return None
+
+        if 'POST' in options_page.json['actions']:
+            post_fields = options_page.json['actions']['POST']
+        else:
+            log.warning("Insufficient privileges on %s, inferring POST fields from description.", options_page.endpoint)
+            post_fields = utils.parse_description(options_page.json['description'])
+        self.post_fields[key] = post_fields
+
+        return post_fields
+
     def set_page(self, page):
         log.debug("set_page: %s %s", type(page), page.endpoint)
         self.pages_by_url[page.endpoint] = page
